import pandas as pd 
import numpy as np
import glob

class loader_feature:
    def load(self, sheet_name=None, file_type=None):
        try:

            # Reading Excel sheet name
            self.sheet_name = sheet_name or self.sheet_name # if sheet_name is None, use the sheet_name from the __init__ method
            if self.sheet_name is None:
                self.sheet_name = pd.ExcelFile(self.path).sheet_names
                # if execl file has only one sheet, load the sheet name
                if len(self.sheet_name) == 1:
                    self.sheet_name = self.sheet_name[0]
                else:
                    raise TypeError(f"\n\n\nData loaded as dictionary. There might be multiple sheets. Please specify the sheet name. \n Available sheet names: {self.sheet_name} \n")
            
            # Reading file type and loading the file
            self.file_type = file_type or self.file_type # if file_type is None, use the file_type from the __init__ method
            if self.file_type == ".xlsx":
                self.df = pd.read_excel(self.path, sheet_name = self.sheet_name)
            else:
                self.df = pd.read_csv(self.path)

            return self.df
            
        except Exception as e:
            raise

    def load_multiple_csv(self):
        try:
            file_list = [file for file in glob.glob(self.path + f"*{self.file_query}*.csv")]

            if not file_list:
                raise FileNotFoundError(f"No files found with the query: {self.file_query}")

            self.df = pd.concat([pd.read_csv(file) for file in file_list])
            return self.df
        
        except Exception as e:
            print(f"Error loading file: {e}. Please check the file path and query")

    def load_multiple_xlsx(self):
        try:
            file_list = [file for file in glob.glob(self.path + f"*{self.file_query}*.xlsx")]

            if not file_list:
                raise FileNotFoundError(f"No files found with the query: {self.file_query}")

            self.df = pd.concat([pd.read_excel(file) for file in file_list])
            return self.df
        
        except Exception as e:
            print(f"Error loading file: {e}. Please check the file path and query")


import os

class export_feature:
    def export(self, file_type = None, file_name = None):
        file_type = file_type or self.file_type # if file_type is None, use the file_type from the __init__ method
        file_name = file_name or os.path.splitext(os.path.basename(self.path))[0]        
        # Determine the file type and save the DataFrame as subsequent file type
        
        # Checking if self.df exists
        if self.df is None:
            raise ValueError("Please call load() before running export()")

        # Exporting the file
        if file_type == ".xlsx":
            self.df.to_excel(f"{file_name}.xlsx", index = False)
        else:
            self.df.to_csv(f"{file_name}.csv", index = False)
        return



import pandas as pd 
import numpy as np
import os 
import glob 
from .eda_feature import eda_feature
from .loader_feature import loader_feature
from .unique_val_feature import unique_val_feature
from .export_feature import export_feature

class extractor(eda_feature, loader_feature, unique_val_feature, export_feature):
    def __init__(self, path, file_type = None, sheet_name = None, file_query = None):
        
        self.path = path
        self.file_type = file_type
        self.sheet_name = sheet_name
        self.file_query = file_query
        self.df = None
        self.unique = None
    




class eda_feature:
    def eda(self):
        if self.df is None:
            raise ValueError("Please call load() before running eda()")
            
        self.unique = {col: self.df[col].unique() for col in self.df.columns}
        self.missing = self.df.isnull().sum()
        
        output = [
            ("\n\nData preview", self.df.head(10)),
            ("Data description", self.df.describe()),
            ("Data info", self.df.info()),
            ("Data columns", self.df.columns),
            ("Unique values in each column", self.unique),
            ("Missing values", self.missing)
        ]
        
        for description, data in output:
            print(f"{description}:\n{data}\n\n\n")
        
        return


import pandas as pd 
import numpy as np 

class unique_val_feature:
    def print_unique(self):
        for column, unique_value in self.unique.items():
            print(f"\nUnique values in {column}: \n {unique_value}\n\n")
        return

    def columns(self):
        print(self.df.columns) 
        return
    
    def unique_values(self, col_name = None):
        col_name = col_name
        if col_name is None:
            raise TypeError("Please specify the column name that you want to retrieve unique values from")  
        try:
            print (f"Unique values in {col_name} \n {self.unique[col_name]}")
            return
        except KeyError:
            print(f"Column '{col_name}' does not exist. Please provide a valid column name.")